/*#============================================================================
 *# Copyright (C) 1989-1991 Recital Corporation Inc.
 *# As an unpublished licenced proprietary work.
 *# All rights reserved worldwide.
 *#============================================================================
 *
 *	MODULE		: dbapi.h
 *
 *	PURPOSE		: Recital/API header file.
 *
 *	AUTHOR		: Recital Corporation Inc.
 *
 *	DATE		: 29-may-1991
 *
 *==========================================================================*/

#include <sys/types.h>

/*=============================================================================
 * API constants
 *===========================================================================*/

#ifndef EXPORT
#ifdef	VAXVMS
#ifdef	VAXC
#define	EXPORT	globaldef noshare
#else
#define	EXPORT	extern
#endif
#else
#define EXPORT
#endif
#endif

#ifndef IMPORT
#ifdef	VAXC
#define	IMPORT	globalref
#else
#define	IMPORT	extern
#endif
#endif

/* data types returned by api_parinfo() */
#define	API_UTYPE	0	/* undefined				*/
#define	API_CTYPE	1	/* character				*/
#define	API_NTYPE	2	/* numeric				*/
#define	API_LTYPE	4	/* logical				*/
#define	API_DTYPE	8	/* date					*/
#define	API_MTYPE	16	/* memo				 	*/
#define	API_REFPAR	32	/* passed by reference with @		*/
#define	API_TTYPE	64	/* datetime				*/
#define	API_YTYPE	128	/* currency				*/
#define	API_OTYPE	256	/* object				*/
#define	API_ATYPE	512	/* array				*/

/* general constants */
#define	API_TOP		-1	/* DBF_GOTO( API_TOP )			*/
#define	API_BOTTOM	-2	/* DBF_GOTO( API_BOTTOM )		*/
#define	API_PUBLIC	0	/* PUBLIC memvars			*/
#define API_PRIVATE	1	/* PRIVATE memvars			*/

/* memory variable data types */
#ifndef DATE
#define DATE		unsigned int
#endif
#ifndef MEMO
#define MEMO		off_t
#endif
#ifndef TIME
#define TIME		unsigned int
#endif
#define CHARACTER	char 
#define LOGICAL		int
#define NUMERIC		double
#ifndef CURRENCY
struct  RCT_CURRENCY_DEF {
	unsigned short int data[4] ;
};
#define CURRENCY struct RCT_CURRENCY_DEF
#endif
#ifndef DATETIME
struct  RCT_DATETIME_DEF {
	DATE	date;
	TIME	time;
};

#define DATETIME struct RCT_DATETIME_DEF
#endif

#define OBJECT		void *
#define OBJARG		char *

#define	API_FUNCTION	0
#define	API_CLASS	1

/*=============================================================================
 * API data structures
 *===========================================================================*/

struct	API_FUNCTION_TABLE {
	char	*name;		/* pointer to recital function name	*/
	int	(*address)();	/* address of C support function	*/
};

struct	API_SHARED_FUNCTION_TABLE {
	char	*name;		/* pointer to recital function name	*/
	char	*apiname;	/* pointer to C support function name	*/
	int	type;		/* function type 0 function 1 class	*/
};

#ifdef errno
#undef errno
#endif

struct API_EXPRESSION {
	int	errno;		/* Recital error() number	     	*/
	char	type;		/* data type of result		     	*/
				/* C	= character or memo	     	*/
				/* N 	= numeric   			*/
				/* D	= date			     	*/
				/* L	= logical		     	*/
				/* T	= datetime		     	*/
				/* Y	= currency		     	*/
	int	width;		/* width of result		     	*/
	int	decimal;	/* decimal places in result	     	*/
	char	*character;	/* CHARACTER result		     	*/
	double	number;		/* NUMERIC result		     	*/
	char	logical;	/* LOGICAL result		     	*/
	DATE    date;		/* DATE result			     	*/
	DATETIME datetime;	/* DATETIME result			*/
	CURRENCY currency;	/* CURRENCY result			*/
};

union 	API_UNION {
	char		*info_character; /* character memory variable  	*/
	char		info_logical;    /* logical memory variable	*/
	double		info_number;     /* numeric memory variable	*/
	DATE		info_date;       /* date memory variable	*/
	DATETIME	info_datetime;	 /* datetime memory variable	*/
	CURRENCY        info_currency;   /* currency memory variable	*/
};

struct	API_MEMVAR {
	char	type;		/* data type of memory variable		*/
	union	API_UNION value;/* value of memory variable		*/
	int	width;		/* display width			*/
	int	decimal;	/* display # of decimal places		*/
};

struct	API_FIELD {
	int	fieldno;	/* field number				*/
	char	type;		/* data type of field			*/
	int	size;		/* storage size in the record		*/
	int	width;		/* display width			*/
	int	decimal;	/* number of decimal places for numerics*/
	char	*value;		/* pointer to data in current record	*/
};

/*=============================================================================
 * API level 1 functions 
 *===========================================================================*/
extern	char			*api_par_c();
extern	char			*api_par_ds();
#ifdef VER90
extern	char			*api_par_ts();
extern	char			*api_par_ys();
#endif
extern	int			api_par_l();
extern	int			api_par_ni();
extern	long			api_par_nl();
extern	double			api_par_nd();
extern	int			api_par_clen();
extern	int			api_par_csiz();
extern	int			api_par_infa();
extern	int			api_par_info();
#ifdef VER90
extern	OBJECT			api_par_o();
#endif
extern	void			api_ret_c();
extern	void			api_ret_clen();
extern	void			api_ret_ds();
#ifdef VER90
extern	void			api_ret_ts();
extern	void			api_ret_ys();
#endif
extern	void			api_ret_l();
extern	void			api_ret_ni();
extern	void			api_ret_nl();
extern	void			api_ret_nd();
#ifdef VER90
extern	void			api_ret_o();
#endif
extern	void			api_ret(); 
#define	_parc			api_par_c
#define	_pards			api_par_ds
#ifdef VER90
#define	_parts			api_par_ts
#define	_parys			api_par_ys
#endif
#define	_parl			api_par_l
#define	_parni			api_par_ni
#define	_parnl			api_par_nl
#define	_parnd			api_par_nd
#define	_parclen		api_par_clen
#define _parcsiz		api_par_csiz
#define _parinfa		api_par_infa
#define _parinfo		api_par_info
#ifdef VER90
#define	_paro			api_par_o
#endif
#define _exmgrab(ARG)		malloc(ARG)
#define _exmback(ARG,LEN)	free(ARG)
#define _xgrab(ARG)		malloc(ARG)
#define _xalloc(ARG)		malloc(ARG)
#define _xfree(ARG,LEN)		free(ARG)
#define	_retc(ARG)		{api_ret_c(ARG);return;}
#define _retclen(ARG,LEN)	{api_ret_clen(ARG,LEN);return;}
#define _retds(ARG)		{api_ret_ds(ARG);return;}
#define _retdt(ARG)		{api_ret_dt(ARG);return;}
#define _retdy(ARG)		{api_ret_dy(ARG);return;}
#define _retl(ARG)		{api_ret_l(ARG);return;}
#define _retni(ARG)		{api_ret_ni(ARG);return;}
#define _retnl(ARG)		{api_ret_nl(ARG);return;}
#define _retnd(ARG)		{api_ret_nd(ARG);return;}
#define	_ret			{api_ret();return;}
#define	PCOUNT			(api_par_info(0))
#define ISCHAR(ARG)		((api_par_info(ARG)&~API_REFPAR)==API_CTYPE)
#define ISNUM(ARG)		((api_par_info(ARG)&~API_REFPAR)==API_NTYPE)
#define ISLOG(ARG)		((api_par_info(ARG)&~API_REFPAR)==API_LTYPE)
#define ISDATE(ARG)		((api_par_info(ARG)&~API_REFPAR)==API_DTYPE)
#define ISMEMO(ARG)		((api_par_info(ARG)&~API_REFPAR)==API_MTYPE)
#define ISBYREF(ARG)		((api_par_info(ARG)&API_REFPAR)
#define ISARRAY(ARG)		((api_par_info(ARG)&~API_REFPAR)==API_ATYPE)
#define ISDATETIME(ARG)		((api_par_info(ARG)&~API_REFPAR)==API_TTYPE)
#define ISCURRENCY(ARG)		((api_par_info(ARG)&~API_REFPAR)==API_YTYPE)
#define ISOBJECT(ARG)		((api_par_info(ARG)&~API_REFPAR)==API_OTYPE)
#define ALENGTH(ARG)		(api_par_infa(ARG,0))

/*=============================================================================
 * API level 2 functions 
 *===========================================================================*/
extern	int			api_command();
extern	int			api_expression();
#ifdef VER90
extern	void			api_error();
#endif
#define COMMAND			api_command
#define	EXPRESSION		api_expression
#ifdef ERROR
#undef ERROR
#endif
#ifdef VER90
#define	ERROR			api_error
#endif
/*=============================================================================
 * API level 3 functions 
 *===========================================================================*/

/* database functions */
extern	int			api_dbf_fetch();
extern	int			api_dbf_read();
extern	int			api_dbf_recall();
extern	int			api_dbf_delete();
extern	int			api_dbf_update();
extern	int			api_dbf_append();
extern	int			api_dbf_seek();
extern	int			api_dbf_lockf();
extern	int			api_dbf_unlockf();
extern	int			api_dbf_lockr();
extern	int			api_dbf_unlockr();
extern	int			api_dbf_scatter();
extern	int			api_dbf_gather();
extern	int			api_dbf_goto();
extern	int			api_dbf_skip();
extern	char			*api_dbf_recbuffer();
extern	char			*api_dbf_alias();
extern	char			*api_dbf_dbf();
extern	char			*api_dbf_filter();
extern	char			*api_dbf_fmt();
extern	int			api_dbf_isexclusive();
extern	int			api_dbf_isread_only();
extern	char			*api_dbf_ndx();
extern	int			api_dbf_reccount();
extern	int			api_dbf_recsize();
extern	int			api_dbf_recno();
extern	int			api_dbf_select();
extern	int			api_dbf_used();
extern	int			api_dbf_isdeleted();
extern	int			api_dbf_isbof();
extern	int			api_dbf_iseof();
extern	int			api_dbf_indexorder();
extern	char			*api_dbf_indexkey();
#define DBF_FETCH		api_dbf_fetch
#define	DBF_READ		api_dbf_read
#define DBF_RECALL		api_dbf_recall
#define DBF_DELETE		api_dbf_delete
#define DBF_DELETED		api_dbf_isdeleted
#define DBF_UPDATE		api_dbf_update
#define DBF_APPEND		api_dbf_append
#define DBF_SEEK		api_dbf_seek
#define DBF_LOCKF		api_dbf_lockf
#define DBF_UNLOCKF		api_dbf_unlockf
#define DBF_LOCKR		api_dbf_lockr
#define DBF_UNLOCKR		api_dbf_unlockr
#define	DBF_RECNO		api_dbf_recno
#define	DBF_RECCOUNT		api_dbf_reccount
#define	DBF_RECSIZE		api_dbf_recsize
#define	DBF_RECBUFFER		api_dbf_recbuffer
#define	DBF_FILTER		api_dbf_filter
#define	DBF_DBF			api_dbf_dbf
#define DBF_ALIAS		api_dbf_alias
#define DBF_SELECT		api_dbf_select
#define	DBF_ISREADONLY		api_dbf_isread_only
#define DBF_ISEXCLUSIVE		api_dbf_isexclusive
#define	DBF_ISBOF		api_dbf_isbof
#define DBF_ISEOF		api_dbf_iseof
#define DBF_FMT			api_dbf_fmt
#define DBF_NDX			api_dbf_ndx
#define DBF_USED		api_dbf_used
#define	DBF_INDEXORDER		api_dbf_indexorder
#define	DBF_INDEXKEY		api_dbf_indexkey
#define DBF_SKIP		api_dbf_skip
#define DBF_GOTO		api_dbf_goto
#define DBF_SCATTER		api_dbf_scatter
#define DBF_GATHER		api_dbf_gather

/* memo functions */
extern	int			api_memo_read();
extern	MEMO			api_memo_write();
extern	MEMO			api_memo_update();
extern	MEMO			api_memo_reclaim();
extern	char			*api_memo_mline();
extern	int			api_memo_mlcount();
extern	int			api_memo_size();
#define MEMO_READ		api_memo_read
#define	MEMO_WRITE		api_memo_write
#define MEMO_UPDATE		api_memo_update
#define MEMO_RECLAIM		api_memo_reclaim
#define MEMO_MLINE		api_memo_mline
#define	MEMO_MLCOUNT		api_memo_mlcount
#define MEMO_SIZE		api_memo_size

/* blob functions */
extern	int			api_blob_read();
extern	MEMO			api_blob_write();
extern	MEMO			api_blob_update();
extern	MEMO			api_blob_reclaim();
extern	int			api_blob_size();
#define	BLOB_READ		api_blob_read
#define BLOB_WRITE		api_blob_write
#define BLOB_UPDATE		api_blob_update
#define BLOB_RECLAIM		api_blob_reclaim
#define BLOB_SIZE		api_blob_size

/* memory variable functions */
extern	struct API_MEMVAR	*api_memvar_lookup();
extern	struct API_MEMVAR	*api_memvar_define();
extern	int			api_memvar_update();
#define	MEMVAR_LOOKUP		api_memvar_lookup
#define MEMVAR_DEFINE		api_memvar_define
#define MEMVAR_UPDATE		api_memvar_update

/* array variable functions */
extern	struct API_MEMVAR	*api_array_lookup();
extern	char			*api_array_define();
extern	int			api_array_update();
extern	int			api_array_alen();
#define ARRAY_LOOKUP		api_array_lookup
#define ARRAY_DEFINE		api_array_define
#define ARRAY_UPDATE		api_array_update
#define ARRAY_ALEN		api_array_alen

/* field variable functions */
extern	struct API_FIELD	*api_field_lookup();
extern	char			*api_field_name();
extern	int			api_field_value();
extern	int			api_field_update();
extern	int			api_field_count();
#define FIELD_LOOKUP		api_field_lookup
#define	FIELD_NAME		api_field_name
#define FIELD_VALUE		api_field_value
#define	FIELD_COUNT		api_field_count
#define FIELD_UPDATE		api_field_update

/* date functions */
extern	char			*api_date_cmonth();
extern	char			*api_date_cdow();
extern	DATE			api_date_ctod();
extern	int			api_date_dow();
extern	char			*api_date_dtoc();
extern	int			api_date_day();
extern	int			api_date_month();
extern	int			api_date_year();
extern	char			*api_date_dtos();
extern	DATE			api_date_stod();
#ifdef VER90
extern	char			*api_date_ttos();
extern	DATETIME		*api_date_stot();
#endif
extern	char			*api_date_date();
extern	char			*api_date_time();
extern	char			*api_date_ampm();
#ifdef VER90
extern	char			*api_date_datetime();
#endif
#define	DATE_CMONTH		api_date_cmonth
#define	DATE_CDOW		api_date_cdow
#define	DATE_CTOD		api_date_ctod
#define	DATE_DOW		api_date_dow
#define	DATE_DTOC		api_date_dtoc
#define	DATE_DAY		api_date_day
#define	DATE_MONTH		api_date_month
#define	DATE_YEAR		api_date_year
#define	DATE_DTOS		api_date_dtos
#define	DATE_STOD		api_date_stod
#ifdef VER90
#define	DATE_TTOS		api_date_ttos
#define	DATE_STOT		api_date_stot
#endif
#define	DATE_DATE		api_date_date
#define	DATE_TIME		api_date_time
#define DATE_AMPM		api_date_ampm
#ifdef VER90
#define	DATE_DATETIME		api_date_datetime
#endif

/* character string functions */
extern	char			*api_char_ltrim();
extern	char			*api_char_rtrim();
extern	char			*api_char_alltrim();
extern	char			*api_char_lpad();
extern	char			*api_char_rpad();
extern	char			*api_char_upper();
extern	char			*api_char_lower();
extern	char			*api_char_str();
#define	CHAR_LTRIM		api_char_ltrim
#define	CHAR_RTRIM		api_char_rtrim
#define	CHAR_ALLTRIM		api_char_alltrim
#define	CHAR_LPAD		api_char_lpad
#define	CHAR_RPAD		api_char_rpad
#define	CHAR_UPPER		api_char_upper
#define	CHAR_LOWER		api_char_lower
#define CHAR_STR		api_char_str

/* currency string functions */
#ifdef VER90
extern	char			*api_date_ytos();
extern	CURRENCY		*api_date_stoy();
#define	CURR_YTOS		api_date_ytos
#define	CURR_STOY		api_date_stoy
#endif
/*=============================================================================
 * API level 4 functions
 *===========================================================================*/

/* native 'C' classes API 

   Class prototype in $(DB_CLASSDIR)/recital/examples/cExample.cop:

	CLASS myclass FUNCTION 'myclass' LIB 'myclasslib'

   Reference in 4GL as:

	import recital.examples.cExample
	or
	import	recital.examples.*


	x = NEW myclass();
*/
	
#define	GETPROPERTY	    1
#define	SETPROPERTY	    2
#define METHODCALL	    3
#define COUNTPROPERTIES	    4
#define COUNTMETHODS	    5
#define COUNTEVENTS	    6
#define QUERYPROPERTIES	    7
#define QUERYMETHODS	    8
#define QUERYENEVENTS	    9
#define UPDATEPROPERTIES    10

extern	void			api_setobject_data(OBJECT object, char *data);
extern	char			*api_getobject_data(OBJECT object);
extern	void			api_count_properties(char	*argv[],char	**properties);
extern	int			api_query_properties(OBJECT object,
						    char    **local_properties,
						    char    *property_argc,
						    char    **prop_names,    
						    char    **prop_desc);  
extern	void			api_count_methods(char	*argv[],char	**methods);
extern	int			api_query_methods(OBJECT    object,
						    char    **local_methods,
						    char    *method_argc,
						    char    **meth_names,    
						    char    **meth_desc);  
extern	int			api_update_properties(OBJECT object,
						    char    *property_argc,
						    char    **prop_names,    
						    char    **prop_value);  
extern	OBJECT			api_object_new(char *object_name, char  *class_name, char	*parameters);
extern  OBJECT			api_object_assign(OBJECT objptr,char *object_name);
extern	void			api_object_delete(OBJECT object);
extern	int			api_object_setproperty(OBJECT object, char *property_name, char	*value);
extern	OBJARG			api_object_getproperty(OBJECT object, char *property_name, char	*buffer);
extern	OBJARG			api_object_method(OBJECT object, char *method_name, char	*args);
extern	int			api_object_getarg(OBJARG objarg, struct API_EXPRESSION *buffer);
extern	OBJARG			api_object_setarg(OBJARG objarg, struct API_EXPRESSION *buffer);


#define	_api_setobject_data	api_setobject_data
#define	SET_OBJECTDATA		_api_setobject_data
#define	_api_getobject_data	api_getobject_data
#define	GET_OBJECTDATA		_api_getobject_data
#define	_api_count_properties	api_count_properties
#define COUNT_PROPERTIES(E)	_api_count_properties(argv, E);
#define	_api_query_properties	api_query_properties
#define QUERY_PROPERTIES(E)	_api_query_properties(object, E,argv[0],(char **)argv[1],(char **)argv[2]);
#define	_api_count_methods	api_count_methods
#define COUNT_METHODS(E)	_api_count_methods(argv, E);
#define	_api_query_methods	api_query_methods
#define QUERY_METHODS(E) _api_query_methods(object, E,argv[0],(char **)argv[1],(char **)argv[2]);
#define	_api_update_properties	api_update_properties
#define UPDATE_PROPERTIES()	_api_update_properties(object, argv[0],(char **)argv[1],(char **)argv[2]);
#define	_api_object_new		api_object_new
#define	OBJECT_NEW		_api_object_new
#define	_api_object_assign	api_object_assign
#define	OBJECT_ASSIGN		_api_object_assign
#define	_api_object_delete	api_object_delete
#define	OBJECT_DELETE		_api_object_delete
#define	_api_object_setproperty	api_object_setproperty
#define	OBJECT_SETPROPERTY	_api_object_setproperty
#define	_api_object_getproperty	api_object_getproperty
#define	OBJECT_GETPROPERTY	_api_object_getproperty
#define	_api_object_method	api_object_method
#define	OBJECT_METHOD		_api_object_method
#define	_api_object_getarg	api_object_getarg
#define	OBJECT_GETARG		_api_object_getarg
#define	_api_object_setarg	api_object_setarg
#define	OBJECT_SETARG		_api_object_setarg


/*----------------------------*/
/* API function address table */
/*----------------------------*/
struct	API_FUNCTION_ADDRESS_TABLE {
	char	*(*api_par_c)();
	char	*(*api_par_ds)();
#ifdef VER90
	char	*(*api_par_ts)();
	char	*(*api_par_ys)();
#endif
	int	(*api_par_l)();	
	int	(*api_par_ni)();
	long	(*api_par_nl)();	
	double	(*api_par_nd)();
	int	(*api_par_clen)();	
	int	(*api_par_csiz)();	
	int	(*api_par_infa)();	
	int	(*api_par_info)();	
#ifdef VER90
	OBJECT	(*api_par_o)();
#endif
	void	(*api_ret_c)();	
	void	(*api_ret_clen)();	
	void	(*api_ret_ds)();
#ifdef VER90
	void	(*api_ret_ts)();	
	void	(*api_ret_ys)();
#endif
	void	(*api_ret_l)();	
	void	(*api_ret_ni)();	
	void	(*api_ret_nl)();
	void	(*api_ret_nd)();	
#ifdef VER90
	void	(*api_ret_o)();	
#endif
	void	(*api_ret)();
#ifdef VER904
	int	(*api_parse_parameters)();
#endif
	int	(*api_command)();
	int	(*api_expression)();
#ifdef VER90
	void	(*api_error)();
#endif
	int	(*api_dbf_fetch)();
	int	(*api_dbf_read)();
	int	(*api_dbf_recall)();
	int	(*api_dbf_delete)();
	int	(*api_dbf_update)();
	int	(*api_dbf_append)();
	int	(*api_dbf_seek)();
	int	(*api_dbf_lockf)();
	int	(*api_dbf_unlockf)();
	int	(*api_dbf_lockr)();
	int	(*api_dbf_unlockr)();
	int	(*api_dbf_scatter)();
	int	(*api_dbf_gather)();
	int	(*api_dbf_goto)();
	int	(*api_dbf_skip)();
	char	*(*api_dbf_recbuffer)();
	char	*(*api_dbf_alias)();
	char	*(*api_dbf_dbf)();
	char	*(*api_dbf_filter)();
	char	*(*api_dbf_fmt)();
	int	(*api_dbf_isexclusive)();
	int	(*api_dbf_isread_only)();
	char	*(*api_dbf_ndx)();
	int	(*api_dbf_reccount)();
	int	(*api_dbf_recsize)();
	int	(*api_dbf_recno)();
	int	(*api_dbf_select)();
	int	(*api_dbf_used)();
	int	(*api_dbf_isdeleted)();
	int	(*api_dbf_isbof)();
	int	(*api_dbf_iseof)();
	int	(*api_dbf_indexorder)();
	char	*(*api_dbf_indexkey)();
	int	(*api_memo_read)();
	MEMO	(*api_memo_write)();
	MEMO	(*api_memo_update)();
	MEMO	(*api_memo_reclaim)();
	char	*(*api_memo_mline)();
	int	(*api_memo_mlcount)();
	int	(*api_memo_size)();
	int	(*api_blob_read)();
	MEMO	(*api_blob_write)();
	MEMO	(*api_blob_update)();
	MEMO	(*api_blob_reclaim)();
	int	(*api_blob_size)();
	struct API_MEMVAR	*(*api_memvar_lookup)();
	struct API_MEMVAR	*(*api_memvar_define)();
	int	(*api_memvar_update)();
	struct API_MEMVAR	*(*api_array_lookup)();
	char	*(*api_array_define)();
	int	(*api_array_update)();
	int	(*api_array_alen)();
	struct API_FIELD	*(*api_field_lookup)();
	char	*(*api_field_name)();
	int	(*api_field_value)();
	int	(*api_field_update)();
	int	(*api_field_count)();
	char	*(*api_date_cmonth)();
	char	*(*api_date_cdow)();
	DATE	(*api_date_ctod)();
	int	(*api_date_dow)();
	char	*(*api_date_dtoc)();
	int	(*api_date_day)();
	int	(*api_date_month)();
	int	(*api_date_year)();
	char	*(*api_date_dtos)();
	DATE	(*api_date_stod)();
#ifdef VER90
	char	*(*api_date_ttos)();
	DATETIME*(*api_date_stot)();
#endif
	char	*(*api_date_date)();
	char	*(*api_date_time)();
	char	*(*api_date_ampm)();
#ifdef VER90
	char	*(*api_date_datetime)();
#endif
	char	*(*api_char_ltrim)();
	char	*(*api_char_rtrim)();
	char	*(*api_char_alltrim)();
	char	*(*api_char_lpad)();
	char	*(*api_char_rpad)();
	char	*(*api_char_upper)();
	char	*(*api_char_lower)();
	char	*(*api_char_str)();
#ifdef VER90
	char	*(*api_date_ytos)();
	CURRENCY*(*api_date_stoy)();
#endif
	void	(*api_setobject_data)(OBJECT object, char	*data);
	char	*(*api_getobject_data)(OBJECT object);
	void	(*api_count_properties)(char	*argv[],char	**properties);
	int	(*api_query_properties)(OBJECT	object,
					char    **local_properties,
					char    *property_argc,
					char    **prop_names,    
					char    **prop_desc);  
	void	(*api_count_methods)(char	*argv[],char	**methods);
	int	(*api_query_methods)(OBJECT	object,
					char    **local_methods,
					char    *method_argc,
					char    **meth_names,    
					char    **meth_desc);  
	int	(*api_update_properties)(OBJECT	object,
					char    *property_argc,
					char    **prop_names,    
					char    **prop_values);  
	OBJECT	(*api_object_new)(char	*object_name, char	*class_name, char   *parameters);
	OBJECT	(*api_object_assign)(OBJECT objptr,char *object_name);
	void	(*api_object_delete)(OBJECT object);
	int	(*api_object_setproperty)(OBJECT object, char *property_name, char	*value);
	OBJARG	(*api_object_getproperty)(OBJECT object, char *property_name, char	*buffer);
	OBJARG	(*api_object_method)(OBJECT object, char *method_name, char	*args);
	int	(*api_object_getarg)(OBJARG objarg, struct API_EXPRESSION *buffer);
	OBJARG	(*api_object_setarg)(OBJARG objarg, struct API_EXPRESSION *buffer);

};

