#include  <stdio.h>
#include  <stdlib.h>

#include "dbapi.h"
#include "Samples.h"

// Add all functions to this structure as follows:
//    Recital Name,    C Function Name
//
// Make sure the last entry is NULL.
//
static	struct	API_SHARED_FUNCTION_TABLE api_function_table[7] = {
        {"schar",   "fnSamplesCharacter",   API_FUNCTION},
        {"stype",   "fnSamplesType",	    API_FUNCTION},
        {"slog",    "fnSamplesLogical",	    API_FUNCTION},
        {"snum",    "fnSamplesNumeric",	    API_FUNCTION},
        {"sopen",   "fnSamplesOpen",	    API_FUNCTION},
        {"myclass", "clsMyClass",	    API_CLASS},
        {NULL,	    NULL,		    -1}
} ;

// Recital API function address table
INIT_API


//---------------------------------------------------------------------
//
// Recital Universal Application Server API Sample functions
//
//---------------------------------------------------------------------

// This is an example of passing a character parameter and returning one.
void fnSamplesCharacter(void)
{
	_retc(_parc(1));
}


// This is an example of passing a character parameter and returning one.
void fnSamplesNumeric(void)
{
	_retni(_parni(1));
}


// This is an example returns the objectData type of the parameter passed.
void fnSamplesType(void)
{
	char	result[10];

	switch (_parinfo(1)) {
	case API_CTYPE:
	    strcpy(result, "Character");
	    break;
	case API_NTYPE:
	    strcpy(result, "Numeric");
	    break;
	case API_LTYPE:
	    strcpy(result, "Logical");
	    break;
	case API_DTYPE:
	    strcpy(result, "Date");
	    break;
	case API_TTYPE:
	    strcpy(result, "DateTime");
	    break;
	case API_YTYPE:
	    strcpy(result, "Currency");
	    break;
	case API_ATYPE:
	    strcpy(result, "Array");
	    break;
	default:
	    strcpy(result, "Unkown");
	    break;
	}

	_retc(result);
}


// This is an example returns "True" or Flase.
void fnSamplesLogical(void)
{
	char	result[10];

	if (_parl(1)) strcpy(result, "True");
	else strcpy(result, "False");

	_retc(result);
}


// This example opens a table.
void fnSamplesOpen(void)
{
	if (_parinfo(1) == API_CTYPE) {
	    _retni(COMMAND(_parc(1)));
	} else {
	    _retni(-1);
	}
}



// This is an example class.
// objectData structures for this example are defined in Sample.h 

DEFINE_CLASS(clsMyClass)
{

	/*-------------------------------------*/ 
	/* Dispatch factory methods and return */
	/*-------------------------------------*/
	DISPATCH_FACTORY();
 
	/*---------------------------------*/
	/* Dispatch constructor and return */ 
	/*---------------------------------*/ 
	DISPATCH_METHOD(clsMyClass, Constructor);

	/*--------------------------------*/
	/* Dispatch destructor and return */
	/*--------------------------------*/
	DISPATCH_METHOD(clsMyClass, Destructor); 

	/*-----------------------------------*/
	/* Dispatch DEFINE method and return */
	/*-----------------------------------*/
	DISPATCH_METHOD(clsMyClass, Define);

	/*------------------------------*/
	/* Dispatch SET or GET PROPERTY */
	/* method for property NumValue */
	/* then return.			*/
	/*------------------------------*/
	DISPATCH_PROPSET(clsMyClass, NumValue);
	DISPATCH_PROPGET(clsMyClass, NumValue); 

	/*------------------------------*/
	/* Dispatch SET or GET PROPERTY */
	/* method for property LogValue */
	/* then return.			*/
	/*------------------------------*/
	DISPATCH_PROPSET(clsMyClass, LogValue);
	DISPATCH_PROPGET(clsMyClass, LogValue);

	/*-------------------------------*/
	/* Dispatch SET or GET PROPERTY  */
	/* method for property DateValue */
	/* then return.			 */
	/*-------------------------------*/
	DISPATCH_PROPSET(clsMyClass, DateValue);
	DISPATCH_PROPGET(clsMyClass, DateValue);

	/*-------------------------------*/
	/* Dispatch SET or GET PROPERTY  */
	/* method for property TimeValue */
	/* then return.			 */
	/*-------------------------------*/
	DISPATCH_PROPSET(clsMyClass, TimeValue);
	DISPATCH_PROPGET(clsMyClass, TimeValue);
 
	/*-------------------------------*/
	/* Dispatch SET or GET PROPERTY  */
	/* method for property CurrValue */
	/* then return.			 */
	/*-------------------------------*/
	DISPATCH_PROPSET(clsMyClass, CurrValue);
	DISPATCH_PROPGET(clsMyClass, CurrValue);

	/*-------------------------------*/
	/* Dispatch SET or GET PROPERTY  */
	/* method for property CharValue */
	/* then return.			 */
	/*-------------------------------*/
	DISPATCH_PROPSET(clsMyClass, CharValue); 
	DISPATCH_PROPGET(clsMyClass, CharValue);

	/*------------------------------*/
	/* Dispatch SET or GET PROPERTY */ 
	/* method for property ObjValue */ 
	/* then return.			*/
	/*------------------------------*/
	DISPATCH_PROPSET(clsMyClass, ObjValue);
	DISPATCH_PROPGET(clsMyClass, ObjValue);

	/*-----------------------------------*/
	/* If message not found return error */
	/*-----------------------------------*/
	OBJECT_RETERROR("Unknown message type");
}

/* Define constructor method */
DEFINE_METHOD(clsMyClass, Constructor) 
{
    	struct example_data *objectDataArea;

	/* Allocate memory for objects objectData area */
	objectDataArea = (struct example_data *) malloc(sizeof(struct example_data));
	if (objectDataArea == NULL) return(-1);

	/* Assign the default property values */
	strcpy(objectDataArea->prop_charvalue, "Test API object");
	objectDataArea->prop_numvalue = 15.2827;
	objectDataArea->prop_logvalue = 'F';
	strcpy(objectDataArea->prop_datevalue, DATE_DATE());
	strcpy(objectDataArea->prop_timevalue, DATE_DATETIME());
	strcpy(objectDataArea->prop_currvalue, "15.2827");
	strcpy(objectDataArea->object_name, "APIobject");
	objectDataArea->prop_objvalue = OBJECT_NEW(objectDataArea->object_name, "exception", NULL);

	/* Set the object objectData area */
	OBJECT_SETDATA((char *)objectDataArea);

	return(0);
}
  
/* Define destructor method */
DEFINE_METHOD(clsMyClass, Destructor) 
{
    	struct example_data *objectData = (struct example_data *)OBJECT_GETDATA();

	if (objectData != NULL) {
	    if (objectData->prop_objvalue != NULL) OBJECT_DELETE(objectData->prop_objvalue);
	    free(objectData);
	    objectData = NULL;
	}
	return(0);
}

/* Define define method */
DEFINE_METHOD(clsMyClass, Define) 
{
    	struct example_data *objectData = (struct example_data *)OBJECT_GETDATA();
	struct      API_EXPRESSION result;
	char	    buffer[512];
	int	    rc;

	/* Check the object class */
	OBJECT_GETPROPERTY(objectData->prop_objvalue, "class", buffer);
	rc = OBJECT_GETARG(buffer, &result);
	if (result.errno == 0 && result.type == 'C' &&
	    strcmp(result.character, "Exception") == 0) { 
	    switch (OBJECT_GETARGC()) {
	    case 2: 
		rc = OBJECT_GETPARAMETER(2, &result);
		if (result.errno == 0 && result.type == 'N') {
		    OBJECT_SETARG(buffer, &result);
		    rc = OBJECT_SETPROPERTY(objectData->prop_objvalue, "errorno", buffer);
		}

	    case 1: 
		rc = OBJECT_GETPARAMETER(1, &result);
		if (result.errno == 0 && result.type == 'C') {
		    OBJECT_SETARG(buffer, &result);
		    rc = OBJECT_SETPROPERTY(objectData->prop_objvalue, "message", buffer);
		}
		break;
	    }
	}

	result.type = 'L';
	result.logical = (rc == 0 ? 'T' : 'F');  
	OBJECT_RETRESULT(&result);
}

/* Define get property method */
DEFINE_PROPERTYGET(clsMyClass, NumValue) 
{
    	struct example_data *objectData = (struct example_data *)OBJECT_GETDATA();

	if (objectData == NULL) return(-1);

	OBJECT_RETPROPERTY('N', objectData->prop_numvalue);
}

/* Define get property method */
DEFINE_PROPERTYGET(clsMyClass, LogValue) 
{
    	struct example_data *objectData = (struct example_data *)OBJECT_GETDATA();

	if (objectData == NULL) return(-1);

	OBJECT_RETPROPERTY('L', objectData->prop_logvalue);
}

/* Define get property method */
DEFINE_PROPERTYGET(clsMyClass, DateValue) 
{
    	struct example_data *objectData = (struct example_data *)OBJECT_GETDATA();

	if (objectData == NULL) return(-1);

	OBJECT_RETPROPERTY('D', objectData->prop_datevalue);
}

/* Define get property method */
DEFINE_PROPERTYGET(clsMyClass, TimeValue) 
{
    	struct example_data *objectData = (struct example_data *)OBJECT_GETDATA();

	if (objectData == NULL) return(-1);

	OBJECT_RETPROPERTY('T', objectData->prop_timevalue);
}

/* Define get property method */
DEFINE_PROPERTYGET(clsMyClass, CurrValue) 
{
    	struct example_data *objectData = (struct example_data *)OBJECT_GETDATA();

	if (objectData == NULL) return(-1);

	OBJECT_RETPROPERTY('Y', objectData->prop_currvalue);
}

/* Define get property method */
DEFINE_PROPERTYGET(clsMyClass, CharValue) 
{
    	struct example_data *objectData = (struct example_data *)OBJECT_GETDATA();

	if (objectData == NULL) return(-1);

	OBJECT_RETPROPERTY('C', objectData->prop_charvalue);
}

/* Define get property method */
DEFINE_PROPERTYGET(clsMyClass, ObjValue) 
{
    	struct example_data *objectData = (struct example_data *)OBJECT_GETDATA();

	if (objectData == NULL) return(-1);

	OBJECT_RETPROPERTY('O', objectData->prop_objvalue);
} 


/* Define set property method */
DEFINE_PROPERTYSET(clsMyClass, NumValue) 
{
    	struct example_data *objectData = (struct example_data *)OBJECT_GETDATA();
	struct API_EXPRESSION result;
	int	rc = OBJECT_ERROR;

	OBJECT_GETVALUE(&result);
	if (result.errno == 0 && result.type == 'N') {
	    objectData->prop_numvalue = result.number;
	    rc = OBJECT_SUCCESS;
	}

	return(rc);
}

/* Define set property method */
DEFINE_PROPERTYSET(clsMyClass, LogValue) 
{
    	struct example_data *objectData = (struct example_data *)OBJECT_GETDATA();
	struct API_EXPRESSION result;
	int	rc = OBJECT_ERROR;

	OBJECT_GETVALUE(&result);
	if (result.errno == 0 && result.type == 'L') {
	    objectData->prop_logvalue = result.logical;
	    rc = OBJECT_SUCCESS;
	}

	return(rc);
}

/* Define set property method */
DEFINE_PROPERTYSET(clsMyClass, DateValue) 
{
    	struct example_data *objectData = (struct example_data *)OBJECT_GETDATA();
	struct API_EXPRESSION result;
	int	rc = OBJECT_ERROR;

	OBJECT_GETVALUE(&result);
	if (result.errno == 0 && result.type == 'D') {
	    strcpy(objectData->prop_datevalue, DATE_DTOS(result.date));
	    rc = OBJECT_SUCCESS;
	}

	return(rc);
}

/* Define set property method */
DEFINE_PROPERTYSET(clsMyClass, TimeValue) 
{
    	struct example_data *objectData = (struct example_data *)OBJECT_GETDATA();
	struct API_EXPRESSION result;
	int	rc = OBJECT_ERROR;

	OBJECT_GETVALUE(&result);
	if (result.errno == 0 && result.type == 'T') {
	    strcpy(objectData->prop_timevalue, DATE_TTOS(result.datetime));
	    rc = OBJECT_SUCCESS;
	}

	return(rc); 
}

/* Define set property method */
DEFINE_PROPERTYSET(clsMyClass, CurrValue) 
{
    	struct example_data *objectData = (struct example_data *)OBJECT_GETDATA();
	struct API_EXPRESSION result;
	int	rc = OBJECT_ERROR;

	OBJECT_GETVALUE(&result);
	if (result.errno == 0 && result.type == 'Y') {
	    strcpy(objectData->prop_currvalue, CURR_YTOS(result.currency));
	    rc = OBJECT_SUCCESS;
	}

	return(rc);
}

/* Define set property method */
DEFINE_PROPERTYSET(clsMyClass, CharValue)  
{
    	struct example_data *objectData = (struct example_data *)OBJECT_GETDATA();
	struct API_EXPRESSION result;
	int	rc = OBJECT_ERROR;

	OBJECT_GETVALUE(&result);
	if (result.errno == 0 && result.type == 'C') {
	    strcpy(objectData->prop_currvalue, result.character);
	    rc = OBJECT_SUCCESS;
	}

	return(rc);
}

/* Define set property method */
DEFINE_PROPERTYSET(clsMyClass, ObjValue) 
{
    	struct example_data *objectData = (struct example_data *)OBJECT_GETDATA();
	OBJECT	objvalue;
	int	rc = OBJECT_ERROR;

	if (OBJECT_GETTYPE() == 'O') {
	    objvalue = OBJECT_GETOBJECT(); 
	    objectData->prop_objvalue = OBJECT_ASSIGN(objvalue, objectData->object_name);
	    rc = OBJECT_SUCCESS;
	}

	return(rc);
}

/*=============================================================================
 *
 * D E B U G F
 *
 * Debuging function.
 *
 *===========================================================================*/
static	FILE	*apifp = NULL;

int	debugf(s, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10)
char	*s;
int	arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10;
{
	if (apifp == NULL) {
	    apifp = fopen("api.txt","w");
	}

	fprintf(apifp, s, arg1, arg2, arg3, arg4, arg5, arg6,
		arg7, arg8, arg9);
	fflush(apifp);
	return(0);
}
